@@ -1,30 +1,34 @@
-import warnings
 import beaker.container as container
 from beaker.exceptions import InvalidCacheBackendError
-from beaker.util import coerce_cache_params
 import beaker.util as util
 
 clsmap = {
-          'memory':container.MemoryContainer,
-          'dbm':container.DBMContainer,
-          'file':container.FileContainer,
+          'memory':container.MemoryNamespaceManager,
+          'dbm':container.DBMNamespaceManager,
+          'file':container.FileNamespaceManager,
           }
 
 try:
     import beaker.ext.memcached as memcached
-    clsmap['ext:memcached'] = memcached.MemcachedContainer
+    clsmap['ext:memcached'] = memcached.MemcachedNamespaceManager
 except InvalidCacheBackendError:
     pass
 
 try:
     import beaker.ext.database as database
-    clsmap['ext:database'] = database.DatabaseContainer
+    clsmap['ext:database'] = database.DatabaseNamespaceManager
+except InvalidCacheBackendError:
+    pass
+
+try:
+    import beaker.ext.sqla as sqla
+    clsmap['ext:sqla'] = sqla.SqlaNamespaceManager
 except InvalidCacheBackendError:
     pass
 
 try:
     import beaker.ext.google as google
-    clsmap['ext:google'] = google.GoogleContainer
+    clsmap['ext:google'] = google.GoogleNamespaceManager
 except (InvalidCacheBackendError, SyntaxError):
     pass
 
@@ -33,59 +37,58 @@ class Cache(object):
     def __init__(self, namespace, **kwargs):
         self.namespace = namespace
         self.context = container.ContainerContext()
-        self._containers = {}
+        self._values = {}
         self.kwargs = kwargs
         self.kwargs.setdefault('type', 'memory')
     
     def put(self, key, value, **kwargs):
-        kw = self.kwargs
-        kw.update(kwargs)
-        kw.setdefault('type', 'memory')
-        self._get_container(key, **kw).set_value(value)
+        self._values.pop(key, None)
+        self._get_value(key, **kwargs).set_value(value)
     set_value = put
     
     def get(self, key, **kwargs):
-        kw = self.kwargs.copy()
-        kw.update(kwargs)
-        kwargs.setdefault('type', 'memory')
-        return self._get_container(key, **kw).get_value()
+        return self._get_value(key, **kwargs).get_value()
     get_value = get
     
     def remove_value(self, key, **kwargs):
-        mycontainer = self._get_container(key, **self.kwargs)
+        mycontainer = self._get_value(key, **kwargs)
         if mycontainer.has_current_value():
             mycontainer.clear_value()
 
-    def _get_container(self, key, type, **kwargs):
+    def _get_value(self, key, **kwargs):
         if isinstance(key, unicode):
             key = key.encode('ascii', 'backslashreplace')
-        mycontainer = self._containers.get(key)
-        if not mycontainer:
+            
+        if not kwargs:
+            value = self._values.get(key)
+        else:
+            value = None
+            
+        if not value:
             kw = self.kwargs.copy()
             kw.update(kwargs)
-            mycontainer = clsmap[type](key, self.context, self.namespace, **kw)
-            self._containers[key] = mycontainer
-        return mycontainer
+            type = kw.pop('type')
+            self._values[key] = value = container.Value(key, self.context, self.namespace, clsmap[type], **kw)
+        return value
     
     def _get_namespace(self, **kwargs):
         type = kwargs['type']
-        return self.context.get_namespace_manager(self.namespace, clsmap[type], **kwargs)
+        return self.context.get_namespace(self.namespace, clsmap[type], **kwargs)
     
     def _get_namespaces(self):
         """return a collection of all distinct ``Namespace`` instances 
         referenced by this ``Cache``."""
-        s = util.Set()
-        s.add(self._get_namespace(**self.kwargs))
-        for mycontainer in self._containers.values():
-            s.add(mycontainer.namespacemanager)
-        return s
+        
+        return util.Set(
+            [v.namespacemanager for v in self._values.values()]).\
+            union([self._get_namespace(**self.kwargs)])
     
     def clear(self):
         """clear this Cache's default namespace, as well as any other 
         Namespaces that have been referenced by this Cache."""
         for namespace in self._get_namespaces():
             namespace.remove()
-        self._containers = {}
+        self._values = {}
     
     # public dict interface
     def __getitem__(self, key):
@@ -95,7 +98,7 @@ class Cache(object):
         return self.has_key(key)
     
     def has_key(self, key):
-        mycontainer = self._get_container(key, **self.kwargs)
+        mycontainer = self._get_value(key)
         return mycontainer.has_current_value()
     
     def __delitem__(self, key):
